// Contains class definitions
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
#include "Config.h"
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>



// ***************
//  Here, we define the actual functions



//||||||||||||||||||||||||||||||||||||||||||||||||
//
//  Class Config
//
//||||||||||||||||||||||||||||||||||||||||||||||||



// Constructor for the Config object.
//
Config::Config(void):
   APDialog( NULL, NULL, TRUE, STYLEFLAGS )
{
   QFont dffont( "helvetica", 10 );

   strcpy(cfgfile,getenv("HOME"));
   strcat(cfgfile,"/.qCdrc");
   readCfg();
   createChildren(dffont);
   connectChildren();
}


// Destructor for the Config object.
//
Config::~Config(void)
{
   delete warntxt;
   delete wmboxlbl;
   delete tkboxlbl;
   delete devboxlbl;
   delete ioctlwarntxt;
   delete cdbox;
   delete dbbox;
   delete dbhbar;
   delete closeb;
   delete saveb;
   delete trayb;
   delete ioctlb;
   delete wmdb_b;
   delete tkdb_b;
   delete devfile;
   delete wmdbfile;
   delete tkdbfile;
}


// Other methods.
//
void Config::open(void)
{
   int status;
   
   setActiveICB( hasTray );
   setActiveWMFB( useWMDB );
   setActiveTKFB( useTKDB );
   status = exec();
}


//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//
//  Private methods.
//
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


// Other methods.
//
void Config::readCfg(void)
{
   ifstream ifs;
   char dummy[BIGBUFLEN];

   ifs.open( cfgfile, ios::in | ios::nocreate | ios::binary );
   if (!ifs)
         {
	    cout << "Using config defaults." << endl;
	    hasTray=true;
	    useIoctl=false;
	    useWMDB = useTKDB = false;
	    wmdbname[0]='\0';
	    tkdbname[0]='\0';
	    strcpy(devname,CDROM_DRIVENAME);
         return;
         }
   
   ifs.getline(dummy,BIGBUFLEN-2);
   ifs.getline(dummy,BIGBUFLEN-2);
   ifs >> hasTray;
   ifs.ignore(1,'\n');
   ifs >> useIoctl;
   ifs.ignore(1,'\n');
   ifs.getline(devname,IOBUFLEN-2);
   ifs >> useWMDB;
   ifs.ignore(1,'\n');
   ifs.getline(wmdbname,IOBUFLEN-2);
   ifs >> useTKDB;
   ifs.ignore(1,'\n');
   ifs.getline(tkdbname,IOBUFLEN-2);
   ifs.close();
}


// Creates child objects.
//
void Config::createChildren(QFont df)
{
   warntxt = new QLabel( this, "warntxt" );
   warntxt->setGeometry( 10, 10, 380, 30 );
   warntxt->setFont( df );
   warntxt->setText( "Warning:  This feature is incomplete!  It configures the bare essentials ONLY.  Make sure all filenames you specify are valid!" );
   warntxt->setAlignment( 1057 );
   warntxt->setMargin( -1 );
   
   cdbox = new QFrame( this, "cdbox" );
   cdbox->setGeometry( 10, 50, 380, 100 );
   cdbox->setFrameStyle( 50 );
   
   closeb = new QPushButton( this, "closeb" );
   closeb->setGeometry( 330, 320, 50, 30 );
   closeb->setText( "Close" );
   closeb->setAutoRepeat( FALSE );
   closeb->setAutoResize( FALSE );
   
   saveb = new QPushButton( this, "saveb" );
   saveb->setGeometry( 20, 320, 50, 30 );
   saveb->setText( "Save" );
   saveb->setAutoRepeat( FALSE );
   saveb->setAutoResize( FALSE );
   
   devfile = new QLineEdit( this, "devfile" );
   devfile->setGeometry( 110, 60, 150, 20 );
   devfile->setFont( df );
   devfile->setText( devname );
   devfile->setMaxLength( IOBUFLEN );
   devfile->setEchoMode( QLineEdit::Normal );
   devfile->setFrame( TRUE );
   
   devboxlbl = new QLabel( this, "devboxlbl" );
   devboxlbl->setGeometry( 20, 60, 90, 20 );
   devboxlbl->setFont( df );
   devboxlbl->setText( "CDROM Device:" );
   devboxlbl->setAlignment( 289 );
   devboxlbl->setMargin( -1 );
   
   trayb = new QCheckBox( this, "trayb" );
   trayb->setGeometry( 20, 90, 210, 20 );
   trayb->setFont( df );
   trayb->setText( "CDROM Drive has a retracting CD tray" );
   trayb->setAutoRepeat( FALSE );
   trayb->setAutoResize( FALSE );
   trayb->setChecked( hasTray );
   
   ioctlb = new QCheckBox( this, "ioctlb" );
   ioctlb->setGeometry( 20, 110, 230, 20 );
   ioctlb->setFont( df );
   ioctlb->setText( "CDROM Drive can use the close_tray_ioctl." );
   ioctlb->setAutoRepeat( FALSE );
   ioctlb->setAutoResize( FALSE );
   ioctlb->setChecked( useIoctl );
   
   ioctlwarntxt = new QLabel( this, "ioctlwarntxt" );
   ioctlwarntxt->setGeometry( 250, 90, 130, 45 );
   ioctlwarntxt->setFont( df );
   ioctlwarntxt->setFrameStyle( 33 );
   ioctlwarntxt->setText( "Note:  If you don't know what \"close_tray_ioctl\" is, leave that option off." );
   ioctlwarntxt->setAlignment( 1313 );
   ioctlwarntxt->setMargin( -1 );
   
   dbbox = new QFrame( this, "dbbox" );
   dbbox->setGeometry( 10, 160, 380, 150 );
   dbbox->setFrameStyle( 50 );
   
   wmdb_b = new QCheckBox( this, "wmdb_b" );
   wmdb_b->setGeometry( 20, 170, 240, 20 );
   wmdb_b->setFont( df );
   wmdb_b->setText( "Read and Save WorkMan-style CD databases." );
   wmdb_b->setAutoRepeat( FALSE );
   wmdb_b->setAutoResize( FALSE );
   wmdb_b->setChecked( useWMDB );
   
   tkdb_b = new QCheckBox( this, "tkdb_b" );
   tkdb_b->setGeometry( 20, 240, 220, 20 );
   tkdb_b->setFont( df );
   tkdb_b->setText( "Read and Save Xmcd-style CD databases." );
   tkdb_b->setAutoRepeat( FALSE );
   tkdb_b->setAutoResize( FALSE );
   tkdb_b->setChecked( useTKDB );
   
   dbhbar = new QFrame( this, "dbhbar" );
   dbhbar->setGeometry( 15, 230, 370, 10 );
   dbhbar->setFrameStyle( 36 );
   
   wmboxlbl = new QLabel( this, "wmboxlbl" );
   wmboxlbl->setGeometry( 20, 200, 120, 20 );
   wmboxlbl->setFont( df );
   wmboxlbl->setText( "WorkMan Database File:" );
   wmboxlbl->setAlignment( 289 );
   wmboxlbl->setMargin( -1 );
   
   tkboxlbl = new QLabel( this, "tkboxlbl" );
   tkboxlbl->setGeometry( 20, 270, 130, 20 );
   tkboxlbl->setFont( df );
   tkboxlbl->setText( "Xmcd Database Directory:" );
   tkboxlbl->setAlignment( 289 );
   tkboxlbl->setMargin( -1 );
   
   wmdbfile = new QLineEdit( this, "wmdbfile" );
   wmdbfile->setGeometry( 140, 200, 240, 20 );
   wmdbfile->setFont( df );
   wmdbfile->setMaxLength( IOBUFLEN );
   wmdbfile->setEchoMode( QLineEdit::Normal );
   wmdbfile->setFrame( TRUE );
   wmdbfile->setText( wmdbname );   

   tkdbfile = new QLineEdit( this, "tkdbfile" );
   tkdbfile->setGeometry( 150, 270, 230, 20 );
   tkdbfile->setFont( df );
   tkdbfile->setMaxLength( IOBUFLEN );
   tkdbfile->setEchoMode( QLineEdit::Normal );
   tkdbfile->setFrame( TRUE );
   tkdbfile->setText( tkdbname );   
   
   resize( 400, 360 );
}


// Connects the child objects.
//
void Config::connectChildren(void)
{
   connect( trayb, SIGNAL(toggled(bool)), 
		  SLOT(setActiveICB(bool)) );
   connect( wmdb_b, SIGNAL(toggled(bool)), 
		  SLOT(setActiveWMFB(bool)) );
   connect( tkdb_b, SIGNAL(toggled(bool)), 
		  SLOT(setActiveTKFB(bool)) );

   setActiveICB(useIoctl);
   setActiveWMFB(useWMDB);
   setActiveTKFB(useTKDB);

   connect( trayb, SIGNAL(toggled(bool)), 
		  SLOT(setTrayFlg(bool)) );
   connect( wmdb_b, SIGNAL(toggled(bool)), 
		  SLOT(setWmFlg(bool)) );
   connect( tkdb_b, SIGNAL(toggled(bool)), 
		  SLOT(setTkFlg(bool)) );
   connect( ioctlb, SIGNAL(toggled(bool)), 
		  SLOT(setIoctlFlg(bool)) );

   connect( devfile, SIGNAL(returnPressed()), 
		  SLOT(setStrDev()) );
   connect( wmdbfile, SIGNAL(returnPressed()), 
		  SLOT(setStrWMDB()) );
   connect( tkdbfile, SIGNAL(returnPressed()), 
		  SLOT(setStrTKDB()) );

   connect( saveb, SIGNAL(clicked()), 
		  SLOT(saveCfg()) );
   connect( closeb, SIGNAL(clicked()), 
		  SLOT(accept()) );
}


// Three private slots for activating/deactivating 
//
void Config::setActiveICB(bool estate)
{
   ioctlb->setEnabled(estate);
}
//
//
void Config::setActiveWMFB(bool estate)
{
   wmdbfile->setEnabled(estate);
}
//
//
void Config::setActiveTKFB(bool estate)
{
   tkdbfile->setEnabled(estate);
}


// Sets the public member flags
//
void Config::setTrayFlg(bool val)
{
   hasTray=val;
}
//
//
void Config::setIoctlFlg(bool val)
{
   useIoctl=val;
}
//
//
void Config::setWmFlg(bool val)
{
   useWMDB=val;
}
//
//
void Config::setTkFlg(bool val)
{
   useTKDB=val;
}


// Sets the filenames.
//
void Config::setStrDev(void)
{
   strncpy(devname,devfile->text(),IOBUFLEN-2);
   trayb->setFocus();
}
//
//
void Config::setStrWMDB(void)
{
   strncpy(wmdbname,wmdbfile->text(),IOBUFLEN-2);
   tkdb_b->setFocus();
}
//
//
void Config::setStrTKDB(void)
{
   strncpy(tkdbname,tkdbfile->text(),IOBUFLEN-2);
   devfile->setFocus();
}


// Other methods.
//
void Config::saveCfg(void)
{
   ofstream ofs;

   ofs.open( cfgfile, ios::out | ios::binary );
   if (!ofs)
         {
         cerr << "Config::saveCfg() - Cannot open rc file, '"
                 << cfgfile << "'." << endl;
         return;
         }
   
   ofs << "NOTE:  Do not alter this file!  You will render it" << endl;
   ofs << "unreadable if you do!" << endl;
   ofs << hasTray << endl;
   ofs << useIoctl << endl;
   ofs << devname << endl;
   ofs << useWMDB << endl;
   ofs << wmdbname << endl;
   ofs << useTKDB << endl;
   ofs << tkdbname << endl;
   ofs.close();
}




//||||||||||||||||||||||||||||||||
//
// End
//
